#include <stdio.h>
#include <stdlib.h>
#include <limits.h>
#include <fcntl.h>
#include <string.h>

/***********************************************************
* FUNCTION NAME: getFileName
*
* DESCRIPTION:  gets from stdin the name of a file
*
* ARGUMENT LIST:
* Argument        Type   IO      Description
* ------------- -------- -- --------------------------------
* const char*   message  I  message to use for prompting the user
* RETURN VALUE: char*    O  name of file
*
* CHANGES: -
*************************************************************/
char *getFileName(const char *message)
{
    char buf[PATH_MAX];

    printf(message);
    fgets(buf, sizeof(buf), stdin);
    buf[strlen(buf)-1] = '\0';
    return strdup(buf);
}
/***********************************************************
* FUNCTION NAME: openAndCheck
*
* DESCRIPTION:  opens for reading given file
*
* ARGUMENT LIST:
* Argument        Type   IO      Description
* ------------- -------- -- --------------------------------
* const char*   name     I  name of file to open
* RETURN VALUE: int      O  file descriptor for successfully opened file
*
* CHANGES: -
*************************************************************/
int openAndCheck(const char *name)
{
    int fd = -1; // file descriptor for opened file
    static char buf[1024];

    if (-1 == (fd = open(name, O_RDONLY)))
    {
        sprintf(buf, "Cannot open %s for reading.", name);
        perror(buf);
        exit(1);
    }
    return fd;
}

/***********************************************************
* FUNCTION NAME: readLine
*
* DESCRIPTION:  reads one line form the file given by its descriptor
*
* ARGUMENT LIST:
* Argument        Type   IO      Description
* ------------- -------- -- --------------------------------
* const int     fd        I file descriptor for reading text
* char*         buf       O buffer for read line
* const char*   fName     I name of file read
* RETURN VALUE: int       read operation result
*
* CHANGES: buf buffer area contents
*************************************************************/
int readLine(const int fd, char *buf, const char * fName)
{
    int resOp, i;

    i = 0;
    do
    {
        resOp = read(fd, buf + i, 1);
        if (resOp < 0)
        {
            sprintf(buf, "Problem reading file %s", fName);
            perror(buf);
            break;
        }
        if (*(buf + i) == '\n')
        {
            *(buf + i) ='\0';
            break;
        }
        i++;
    }
    while (resOp > 0);
    if (resOp == 0) *(buf + i) ='\0';
    return resOp;
}
int main()
{
    int fd1, fd2; //file descriptors for files to compare
    char *fName1, *fName2; // file names
    static char buf1[65536], buf2[65536];
    int f1Exhausted = 0, f2Exhausted = 0;

    fName1 = getFileName("The reference file for comparison is: ");
    fName2 = getFileName("The file to compare with is: ");
    fd1 = openAndCheck(fName1);
    fd2 = openAndCheck(fName2);

    do
    {
        f1Exhausted = readLine(fd1, buf1, fName1) == 0;
        f2Exhausted = readLine(fd2, buf2, fName2) == 0;
        if (strcmp(buf1, buf2))
        {
            puts(buf1);
            puts(buf2);
        }
    }
    while (!f1Exhausted && !f2Exhausted);
    if (f1Exhausted && !f2Exhausted)
    {
        printf("%s ended\n", fName1);
        while(readLine(fd2, buf2, fName2)) puts(buf2);
    }
    else if (!f1Exhausted && f2Exhausted)
    {
        printf("%s ended\n", fName2);
        while(readLine(fd1, buf1, fName1)) puts(buf1);
    }

    close(fd1);
    close(fd2);
    return 0;
}
